#!/bin/csh -f
#
# $Header: do_conf.c,v 1.4 89/02/07 10:24:03 phb Exp $
#
# (c) 1988 Apunix Computer Services.  All rights reserved.
#	Dr. Peter H. Berens
#	Apunix Computer Services
#	9921 Carmel Mountain Road, Suite 321
#	San Diego, CA  92129  USA
#	(619) 484-0074
#
# $Log:	do_conf.c,v $
# Revision 1.4  89/02/07  10:24:03  phb
# Added info line and corrected spelling.
# 
# Revision 1.3  88/12/14  12:30:31  phb
# Fixed problem with added device switch entries in SunOS 3.X.
# 
# Revision 1.2  88/11/08  13:09:09  phb
# Added sx driver support.
# 
# Revision 1.1  88/11/08  10:33:05  phb
# Initial revision
# 
# usage: device-name conf.c.file
#
# This script requires super-user privileges.
#

set path = ( . /usr/ucb /usr/bin /bin /etc /usr/etc )

if ( $#argv != 3 ) then
	echo ">>>> Usage: do_conf.c device_prefix  conf.c_pathname SunOS_release"
	exit 1
endif
set dev=$1
set conf=$2
set release=$3
set tmpfile=/tmp/install$$

again:
echo " "
echo " "
echo "[The next step is mandatory for driver installation]"
echo " "
echo -n "About to modify ${conf}. OK? (y/n) "
set ans = $<
if ( $ans != "y") then
	echo " "
	echo "+ Skipping $conf."
	echo " "
	exit 0
endif

echo " "
set delim = '"'
set string = ${delim}"${dev}.h"${delim}
retry:
grep -s ${string} ${conf}
if ($status == 0) then
	echo " "
	echo ">>>> ${dev} already installed in ${conf} <<<<"
	echo " "
	echo ">>>> Several options are available at this point:"
	echo ">>>>	1. Suspend this program, edit out the existing entries in"
	echo ">>>>	   ${conf} manually, and then resume at this point."
	echo ">>>>	2. Assume the existing entries in ${conf} are correct and go on"
	echo ">>>>	   to the next installation phase."
	echo ">>>>	3. Abort the installation."
	echo " "
	getans:
	echo -n  "Please enter the number corresponding to your decision: (1/2/3) "
	set ans = $<
	switch ($ans)
		case "1":
			echo " "
			echo ">>>> Edit ${conf} and then type 'fg' to return to this program."
			echo " "
			suspend
			echo " "
			echo "+ Retrying..."
			echo " "
			goto retry;
		case "2":
			echo " "
			echo "+ Skipping modifications to ${conf}; going on..."
			echo " "
			exit 0;
		case "3":
			echo " "
			echo ">>>> Aborting installation <<<<"
			echo " "
			exit 1;
		default:
			echo " "
			echo ">>>> Please answer with a number <<<<"
			echo " "
			goto getans;
		endsw
endif
echo "+ Beginning to add $dev entries to ${conf}."

#
# set required protections
#

if (! -w ${conf} ) then
	echo "+ Changing protection on ${conf}."
	chmod 644 ${conf}
	if ( $status != 0 ) then
	    echo ">>>> Mode of ${conf} could not be changed <<<<"
	    echo ">>>> Super-user privileges required <<<<"
	    goto bad
	endif
endif

#
# copy current file in case of disaster
#
if ( ! -e ${conf}.no${dev} ) then
	echo "+ Making backup copy of ${conf} as ${conf}.no${dev}."
	cp ${conf} ${conf}.no${dev}
	if ($status != 0) then
		echo " "
		echo ">>>> Backup copy of ${conf} to ${conf}.no${dev} FAILED <<<<"
		echo " "
		goto bad
	endif
else
	echo "+ Backup copy of ${conf} already exists."
endif


#
# Add the define lines to the file
#
#include "${dev}.h"
echo "+ Adding defines to ${conf}."
ed - ${conf} >& /dev/null << @
/bdevsw.*=/i
.
?#endif?a

.
.r proto/${dev}.conf.include.${release}
w
q
@

#
# determine the device number to assign by counting the number of devices
# in bdevsw and cdevsw
#
echo -n "+ Counting block device entries..."
ed - ${conf} << @ | /usr/ucb/wc -l | tr -d " " > ${tmpfile}
/bdevsw.*=/i
.
.,/cdevsw.*=/g/},/.p
q
@
set BDEV=`cat ${tmpfile}`
echo "${BDEV} found."
echo -n "+ Counting character device entries..."
ed - ${conf} << @ | /usr/ucb/wc -l | tr -d " " > ${tmpfile}
/cdevsw.*=/i
.
.,/};/g/},/.p
q
@
set CDEV=`cat ${tmpfile}`
echo "${CDEV} found."

#
# Add in entry, insuring that the previous entry ends with a comma
# and including the device number.
#
if ( -r proto/${dev}.conf.bdev.${release} ) then
	echo "+ Adding bdevsw entry to ${conf}."
	sed "s/??/${BDEV}/" < proto/${dev}.conf.bdev.${release} > ${tmpfile}
	ed - ${conf} >& /dev/null << @
/bdevsw.*=/i
.
/};/p
?}?s/}.*/},/
.r ${tmpfile}
w
q
@
endif
echo "+ Adding cdevsw entry to ${conf}."
sed "s/??/${CDEV}/" < proto/${dev}.conf.cdev.${release} > ${tmpfile}
ed - ${conf} >& /dev/null << @
/cdevsw.*=/i
.
/};/p
?}?s/}.*/},/
.r ${tmpfile}
w
q
@

#
# leaving file with device numbers for later use
#
echo ${CDEV} > /tmp/${dev}.cdev
echo ${BDEV} > /tmp/${dev}.bdev

#
# check for new entry
#
echo "+ Checking for new entries in ${conf}."
grep -s ${dev}.h ${conf}
if ( $status != 0 ) then
	echo ">>>> Device ${dev} configuration addition to ${conf} failed <<<<"
	echo ">>>> Restoring original files <<<<"
	rm -f ${tmpfile}
	cp ${conf}.no${dev} ${conf}
	goto bad
endif
grep -s ${dev}open, ${conf}
if ( $status != 0 ) then
	echo ">>>> Device ${dev} configuration addition to ${conf} failed <<<<"
	echo ">>>> Restoring original files <<<<"
	rm -f ${tmpfile}
	cp ${conf}.no${dev} ${conf}
	goto bad
endif

echo ""
echo "+ Successful ${dev} device description addition to ${conf}"

rm -f ${tmpfile}
exit 0

bad:
echo " "
echo ">>>> Several options are available at this point:"
echo ">>>>	1. Suspend this program, resolve the difficulty in" 
echo ">>>>	   ${conf} manually, and then retry this operation."
echo ">>>>	2. Skip this step and continue with the installation."
echo ">>>>	3. Abort the installation."
echo " "
ga:
echo -n  "Please enter the number corresponding to your decision: (1/2/3) "
set ans = $<
switch ($ans)
	case "1":
		echo " "
		echo ">>>> Resolve problem and then type 'fg' to return to this program."
		echo " "
		suspend
		echo " "
		echo "+ Retrying..."
		goto again;
	case "2":
		echo " "
		echo "+ Skipping modifications to ${conf}; going on..."
		echo " "
		exit 0;
	case "3":
		echo " "
		echo ">>>> Aborting installation <<<<"
		echo " "
		exit 1;
	default:
		echo " "
		echo ">>>> Please answer with a number <<<<"
		echo " "
		goto ga;
	endsw
